unit formPTChart;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Data.DB,
  VCLTee.TeEngine, VCLTee.TeeURL, VCLTee.TeeSeriesTextEd, VCLTee.Series,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.DBChart, VCLTee.TeeTools, commasplit,
  Vcl.StdCtrls, Vcl.buttons;

type
  TfrmPTChart = class(TForm)
    DBChart1: TDBChart;
    Series1: TLineSeries;
    Series2: TLineSeries;
    SeriesTextSource1: TSeriesTextSource;
    SeriesTextSource2: TSeriesTextSource;
    Series3: TLineSeries;
    SeriesTextSource3: TSeriesTextSource;
    Series4: TLineSeries;
    SeriesTextSource4: TSeriesTextSource;
    procedure FormShow(Sender: TObject);
    procedure DBChart1BeforeDrawAxes(Sender: TObject);

  private
    procedure ReadToArrays;
    function Elev(aMD: double): double;
    procedure DrawRightAxis;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPTChart: TfrmPTChart;
  MD: array of double;
  VD: array of double;
  EL: array of double;

implementation

{$R *.dfm}

// ========== TfrmPTChart.FormShow =============================================
procedure TfrmPTChart.FormShow(Sender: TObject);
var
  i: integer;
  LStrList: TStringList;
  Lcs: TCommaSplitter;

begin
// Load Pressure vs Depth (Left Axis)
  Series1.DataSource := SeriesTextSource1;
  with SeriesTextSource1 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

// Load WHP vs Depth (Left Axis)
  Series2.DataSource := SeriesTextSource2;
  with SeriesTextSource2 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',4);
    AddField('Y',1);
    Load;
  end;

// Load Temperature vs Depth (Left Axis)
  Series3.DataSource := SeriesTextSource3;
  with SeriesTextSource3 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',2);
    AddField('Y',1);
    Load;
  end;

// Load Pressure vs Elevation (Right Axis)
  Series4.DataSource := SeriesTextSource4;
  with SeriesTextSource4 do
  begin
    FileName := 'C:\Code_Test_Progs\DeviatedPT\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

  ReadToArrays;
  DrawRightAxis;

end;

// ========== TfrmPTChart.ReadToArrays =========================================
// Read Measured Depth/Vertical Depth/Elevation for well
procedure TfrmPTChart.ReadToArrays;
var
  Lsl: TStringList;
  i: integer;
  s: string;

begin
  Lsl := TStringList.Create;
  try
    Lsl.LoadFromFile('C:\Code_Test_Progs\DeviatedPT\AT-203_Deviation.csv');

    SetLength(MD,Lsl.Count);
    SetLength(VD,Lsl.Count);
    SetLength(EL,Lsl.Count);

    for i := 1 to Lsl.Count - 1 do
    begin
      s := Lsl[i];
      begin
        MD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        s := Copy(s,Pos(',',s) + 1,Length(s));
        VD[i-1] := StrToFloat(Copy(s,1,Pos(',',s)-1));
        EL[i-1] := StrToFloat(Copy(s,Pos(',',s) + 1,Length(s)));
      end;
    end;
  finally
    Lsl.Free;
  end;
end;

// ========== TfrmPTChart.Elev =================================================
// Calculate Elevation for any specified depth
function TfrmPTChart.Elev(aMD: double): double;
var
  i,j: Integer;

begin
  if aMD < MD[0] then
    j := 0
  else
  if aMD > MD[Length(MD)-2] then
    j := Length(MD) - 3
  else
  for i := 1 to Length(MD) - 2 do
  begin
    if aMD < MD[i] then
    begin
      j := i - 1;
      break;
    end;
  end;

  result := (aMD - MD[j])/(MD[j+1] - MD[j])*(EL[j+1]-EL[j]) + EL[j];

end;

// ========== TfrmPTChart.DBChart1BeforeDrawAxes ===============================
// Now works on zoom, scroll etc
procedure TfrmPTChart.DBChart1BeforeDrawAxes(Sender: TObject);
begin
  DrawRightAxis;
end;

// ========== TfrmPTChart.DrawRightAxis ========================================
// Uses left axis increments to locate right axis labels
procedure TfrmPTChart.DrawRightAxis;
var
  i: integer;
  E: double;

begin
  with DBChart1.Axes.Right do
  begin
    Items.Clear;
    i:=Round(Round(DBChart1.Axes.Left.Minimum/DBChart1.Axes.Left.CalcIncrement)*DBChart1.Axes.Left.CalcIncrement);
    repeat
      E := Round(Elev(i));
      Items.Add(i, FloatToStr(E));  // adding custom label
      Inc(i, Round(DBChart1.Axes.Left.CalcIncrement));
    until i>=Round(DBChart1.Axes.Left.Maximum);
  end;

end;

end.
